// Profile.java	last mod:	5/21/97	dshapiro


package w3c.pics.profiler;

import java.io.*;
import java.util.*;
import java.net.URL;
import java.net.MalformedURLException;
import w3c.tools.sexpr.*;

/**
 * An object representing a .pro file.  It contains the state information for 
 * a particular profile.
 **/

public class Profile {

  private boolean DEBUG = false;
  String stringrep = null;
  Vector thepros = new Vector();
  Vector therats = new Vector();
  Vector states = new Vector();
  Vector theurls = new Vector();
  Vector lsources = new Vector();
	Vector theurls2 = new Vector();
	Vector thefiles = new Vector();
	Vector fullnames = new Vector();
	Vector bools = new Vector();
	String filesep = System.getProperty("file.separator");
	Vector sinfos = new Vector();
	String filename = null;
	String dirname = null;
	String passes = null;
	String blocks = null;
	String description = null;
	String sourceURL = null;


  /**
   * Creates a new Profile object.
   * @param filename The name of the .pro file.
   * @param opendir The current working directory.
   **/

  public Profile(String filename, String opendir) {
    FileInputStream fis;
    try
    {
			File testfile = new File(opendir+filesep+"pro_files");
			if (testfile.isDirectory()) {
				testfile = new File(opendir+filesep+"pro_files"+filesep+filename);
				if (testfile.exists()) 
					fis=new FileInputStream(opendir+filesep+"pro_files"+filesep+
					filename);
				else
					fis = new FileInputStream(filename);
			} 
			else {
				fis=new FileInputStream(filename);      
			}
			String thisLine;
			try {
				DataInputStream myInput = new DataInputStream(fis);
				try {	
					if ((thisLine = myInput.readLine()) != null) {
						stringrep = thisLine;
					}
					thisLine = myInput.readLine();
					while ((thisLine !=null)
					 && (!thisLine.startsWith("Begin URLs"))
					 && (!thisLine.startsWith("Label Sources"))) {
						states.addElement(thisLine);
						thisLine = myInput.readLine();
						thefiles.addElement(thisLine);
						thisLine = myInput.readLine();
						fullnames.addElement(thisLine);
						thisLine = myInput.readLine();
					}
					if (thisLine!=null) {
						if (thisLine.startsWith("Begin URLs")) {
							thisLine = myInput.readLine();
							while ((thisLine!=null) && 
										 (!(thisLine.startsWith("End URLs")))) {
								addURL(thisLine);
								thisLine = myInput.readLine();
							}
						thisLine = myInput.readLine();
						}
					}
					if (thisLine!=null) {
						if (thisLine.startsWith("Begin URLs2")) {
							thisLine = myInput.readLine();
							while((thisLine!=null) && 
									(!thisLine.startsWith("End URLs2"))) {
								addURL2(thisLine);
								thisLine = myInput.readLine();
							}
							thisLine = myInput.readLine();
						}
						if (thisLine!=null) {
							if (thisLine.startsWith("Label Sources")) {
								thisLine = myInput.readLine();
								while (thisLine!=null) {
									addLSource(thisLine);
									thisLine = myInput.readLine();
								}
							}
						}
					}
				}
				catch (Exception e) {
					System.out.println("Error: " + e);
				}
				try {
					myInput.close();
					fis.close();
				}
				catch (Exception e) {
					System.out.println("I/O error closing .pro file");
				}
			}
			catch (Exception e) {
				System.out.println("Error: " + e);
			}
    }
    catch (Exception e) {
      System.out.println("failed to open file " + filename);
      System.out.println("Error: " + e);
    }
    if (stringrep!=null)
      tokenize();
  }

  /**
   * Creates a new Profile object.
   * @param filelist The list of .pro and .rat files for this Profile.
   **/

  public Profile(String filelist) {
    
    stringrep = filelist;
    tokenize();
  }

	/**
	 * Creates a new Profile object.
	 * @param filelist The list of .pro and .rat files for this Profile.
	 * @param fname The name of the .pro file for this Profile.
	 * @param dname The name of the directory for this Profile.
	 **/

	public Profile(String filelist, String fname, String dname) {
		
		stringrep = filelist;
		tokenize();
		filename = fname;
		dirname = dname;
	}

	/**
	 * Creates a new Profile object.
	 * @param thelist An enumeration containing the profile information.
	 * @param fname The name of the file for this Profile.
	 * @param dname The name of the directory for this Profile.
	 **/

	public Profile(Enumeration thelist, String fname, String dname) {
		filename = fname;
		dirname = dname;
		stringrep = "";
		while (thelist.hasMoreElements()) {
			String clausename = thelist.nextElement().toString().toLowerCase();
			if (clausename.equals("failurl")) {
			  if (DEBUG) System.out.println("failurl found");
				Vector failed = (Vector)thelist.nextElement();
				Enumeration urlf = failed.elements();
				while (urlf.hasMoreElements()) {
					String theurl = urlf.nextElement().toString();
					addURL(theurl);
					if (DEBUG) System.out.println("failed URL: "+theurl);
				}
				continue;
			}
			else if (clausename.equals("passurl")) {	
				if (DEBUG) System.out.println("passurl found");
				Vector passed = (Vector)thelist.nextElement();
			  Enumeration urlp = passed.elements();
				while (urlp.hasMoreElements()) {
					String theurl = urlp.nextElement().toString();
					addURL2(theurl);
					if (DEBUG) System.out.println("passed URL: "+theurl);
				}
				continue;
			}
			else if (clausename.equals("name")) {
				if (DEBUG) System.out.println("name found");
				Vector nameclause = (Vector)thelist.nextElement();
			  Enumeration nameenum = nameclause.elements();
				while (nameenum.hasMoreElements()) {
					String attribname = nameenum.nextElement().toString().
						toLowerCase();
					if (attribname.equals("rulename")) {
						String rname = nameenum.nextElement().toString();
						if (DEBUG) System.out.println("name:rulename: "+rname);
						continue;
					}
					else if (attribname.equals("description")) {
						String desc = nameenum.nextElement().toString();
						if (DEBUG) System.out.println("name:description: "+desc);
						description = desc;
						continue;
					}
					else {
						nameenum.nextElement();
						if (DEBUG) System.out.println("name: something else found");
						continue;
					}
				}
				continue;
			}
			else if (clausename.equals("source")) {
				if (DEBUG) System.out.println("source found");
				Vector sourceclause = (Vector)thelist.nextElement();
				Enumeration sourceenum = sourceclause.elements();
				while (sourceenum.hasMoreElements()) {
					String attribname = sourceenum.nextElement().toString().
						toLowerCase();
					if (attribname.equals("sourceurl")) {
						String surl = sourceenum.nextElement().toString();
						if (DEBUG) System.out.println("source:sourceURL: "+surl);
						sourceURL = surl;
						continue;
					}
					else if (attribname.equals("creationtool")) {
						String ctool = sourceenum.nextElement().toString();
						if (DEBUG) System.out.println("source:creationTool: "+ctool);
						continue;
					}
					else if (attribname.equals("author")) {
						String author = sourceenum.nextElement().toString();
						if (DEBUG) System.out.println("source:author "+author);
						continue;
					}
					else if (attribname.equals("lastmodified")) {
						String lastmod = sourceenum.nextElement().toString();
						if (DEBUG) System.out.println("source:lastModified "+lastmod);
						continue;
					}
					else {
						sourceenum.nextElement();
						if (DEBUG) System.out.println("source: something else found");
						continue;
					}
				}
				continue;
			}
			else if (clausename.equals("serviceinfo")) {
				if (DEBUG) System.out.println("serviceinfo found");
				Vector infoclause = (Vector)thelist.nextElement();
				Enumeration infoenum = infoclause.elements();
				LabelSource si = new LabelSource();
				sinfos.addElement(si);
				while (infoenum.hasMoreElements()) {
					String infoname = infoenum.nextElement().toString().toLowerCase();
					if (infoname.equals("name")) {
						String iname = infoenum.nextElement().toString();
						si.putName(iname);
						if (DEBUG) System.out.println("serviceinfo:name: "+iname);
						continue;
					}
					else if (infoname.equals("shortname")) {
						String sname = infoenum.nextElement().toString();
						si.putShortname(sname);
						if (DEBUG) System.out.println("serviceinfo:shortname: "+sname);
						addLSource(sname);
						continue;
					}
					else if (infoname.equals("bureauurl")) {
						String burl = infoenum.nextElement().toString();
						si.putBureauURL(burl);
						if (DEBUG) System.out.println("serviceinfo:bureauURL: "+burl);
						continue;
					}
					else if (infoname.equals("bureauunavailable")) {
						String bun = infoenum.nextElement().toString();
						si.putBureauUnavail(bun);
						if (DEBUG) System.out.println("serviceinfo:bureauUnavailable: "+bun);
						continue;
					}
					else if (infoname.equals("ratfile")) {
						String ratfile = infoenum.nextElement().toString();
						si.parseRatfile(ratfile);
						// System.out.println("serviceinfo:ratfile: "+ratfile);
						continue;
					}
					else {
						infoenum.nextElement();
						if (DEBUG) System.out.println("serviceinfo: something else found");
						continue;
					}
				}
				continue;
			}
			else if (clausename.equals("filter")) {
				if (DEBUG) System.out.println("filter found");
				Vector filterclause = (Vector)thelist.nextElement();
				Enumeration filterenum = filterclause.elements();
				while (filterenum.hasMoreElements()) {
					String filtername = filterenum.nextElement().toString().
						toLowerCase();
					if (filtername.equals("pass")) {
						String pass = filterenum.nextElement().toString();
						if (DEBUG) System.out.println("filter:pass: "+pass);
						passes = pass;
						continue;
					}
					else if (filtername.equals("block")) {
						String block = filterenum.nextElement().toString();
						if (DEBUG) System.out.println("filter:block: "+block);
						blocks = block;
						continue;
					}
					else {
						filterenum.nextElement();
						if (DEBUG) System.out.println("filter: something else found");
						continue;
					}
				}
				continue;
			}
			else {
				if (DEBUG) System.out.println("Something else found");
				thelist.nextElement();
				continue;
			}
		}
	}

  public Profile(File thefile) {
    String fname = thefile.getName();
    String dname = thefile.getParent();
    FileInputStream fis;
    try {
      fis = new FileInputStream(thefile);
    }
    catch (Exception ex) {
      System.out.println("Error opening file");
      return;
    }
    SExprStream p = new SimpleSExprStream(fis);
    p.setListsAsVectors(true);
    Object e;
    try {
      e = p.parse();
    }
    catch (Exception ex) {
      System.out.println("SExprParserException in Profile");
      return;
    }
    try {
      fis.close();
    }
    catch (IOException ex) {
      System.out.println("Error closing file in Profile");
    }
    Enumeration vlist = ((Vector)e).elements();
    vlist.nextElement();
    Enumeration thelist = ((Vector)vlist.nextElement()).elements();
    filename = fname;
    dirname = dname;
    stringrep = "";
    while (thelist.hasMoreElements()) {
      String clausename = thelist.nextElement().toString().toLowerCase();
      if (clausename.equals("failurl")) {
	if (DEBUG) System.out.println("failurl found");
	Vector failed = (Vector)thelist.nextElement();
	Enumeration urlf = failed.elements();
	while (urlf.hasMoreElements()) {
	  String theurl = urlf.nextElement().toString();
	  addURL(theurl);
	  if (DEBUG) System.out.println("failed URL: "+theurl);
	}
	continue;
      }
      else if (clausename.equals("passurl")) {	
	if (DEBUG) System.out.println("passurl found");
	Vector passed = (Vector)thelist.nextElement();
	Enumeration urlp = passed.elements();
	while (urlp.hasMoreElements()) {
	  String theurl = urlp.nextElement().toString();
	  addURL2(theurl);
	 if (DEBUG)  System.out.println("passed URL: "+theurl);
	}
	continue;
      }
      else if (clausename.equals("name")) {
	if (DEBUG) System.out.println("name found");
	Vector nameclause = (Vector)thelist.nextElement();
	Enumeration nameenum = nameclause.elements();
	while (nameenum.hasMoreElements()) {
	  String attribname = nameenum.nextElement().toString().
	    toLowerCase();
	  if (attribname.equals("rulename")) {
	    String rname = nameenum.nextElement().toString();
	    if (DEBUG) System.out.println("name:rulename: "+rname);
	    continue;
	  }
	  else if (attribname.equals("description")) {
	    String desc = nameenum.nextElement().toString();
	    if (DEBUG) System.out.println("name:description: "+desc);
	    description = desc;
	    continue;
	  }
	  else {
	    nameenum.nextElement();
	    if (DEBUG) System.out.println("name: something else found");
	    continue;
	  }
	}
	continue;
      }
      else if (clausename.equals("source")) {
	if (DEBUG) System.out.println("source found");
	Vector sourceclause = (Vector)thelist.nextElement();
	Enumeration sourceenum = sourceclause.elements();
	while (sourceenum.hasMoreElements()) {
	  String attribname = sourceenum.nextElement().toString().
	    toLowerCase();
	  if (attribname.equals("sourceurl")) {
	    String surl = sourceenum.nextElement().toString();
	    if (DEBUG) System.out.println("source:sourceURL: "+surl);
	    sourceURL = surl;
	    continue;
	  }
	  else if (attribname.equals("creationtool")) {
	    String ctool = sourceenum.nextElement().toString();
	    if (DEBUG) System.out.println("source:creationTool: "+ctool);
	    continue;
	  }
	  else if (attribname.equals("author")) {
	    String author = sourceenum.nextElement().toString();
	    if (DEBUG) System.out.println("source:author "+author);
	    continue;
	  }
	  else if (attribname.equals("lastmodified")) {
	    String lastmod = sourceenum.nextElement().toString();
	    if (DEBUG) System.out.println("source:lastModified "+lastmod);
	    continue;
	  }
	  else {
	    sourceenum.nextElement();
	    if (DEBUG) System.out.println("source: something else found");
	    continue;
	  }
	}
	continue;
      }
      else if (clausename.equals("serviceinfo")) {
	if (DEBUG) System.out.println("serviceinfo found");
	Vector infoclause = (Vector)thelist.nextElement();
	Enumeration infoenum = infoclause.elements();
	LabelSource si = new LabelSource();
	sinfos.addElement(si);
	while (infoenum.hasMoreElements()) {
	  String infoname = infoenum.nextElement().toString().toLowerCase();
	  if (infoname.equals("name")) {
	    String iname = infoenum.nextElement().toString();
	    si.putName(iname);
	    if (DEBUG) System.out.println("serviceinfo:name: "+iname);
	    continue;
	  }
	  else if (infoname.equals("shortname")) {
	    String sname = infoenum.nextElement().toString();
	    si.putShortname(sname);
	    if (DEBUG) System.out.println("serviceinfo:shortname: "+sname);
	    addLSource(sname);
	    continue;
	  }
	  else if (infoname.equals("bureauurl")) {
	    String burl = infoenum.nextElement().toString();
	    si.putBureauURL(burl);
	    if (DEBUG) System.out.println("serviceinfo:bureauURL: "+burl);
	    continue;
	  }
	  else if (infoname.equals("bureauunavailable")) {
	    String bun = infoenum.nextElement().toString();
	    si.putBureauUnavail(bun);
	    if (DEBUG) System.out.println("serviceinfo:bureauUnavailable: "+bun);
	    continue;
	  }
	  else if (infoname.equals("ratfile")) {
	    String ratfile = infoenum.nextElement().toString();
	    si.parseRatfile(ratfile);
	    // if (DEBUG) System.out.println("serviceinfo:ratfile: "+ratfile);
	    continue;
	  }
	  else {
	    infoenum.nextElement();
	    if (DEBUG) System.out.println("serviceinfo: something else found");
	    continue;
	  }
	}
	continue;
      }
      else if (clausename.equals("filter")) {
	if (DEBUG) System.out.println("filter found");
	Vector filterclause = (Vector)thelist.nextElement();
	Enumeration filterenum = filterclause.elements();
	while (filterenum.hasMoreElements()) {
	  String filtername = filterenum.nextElement().toString().
	    toLowerCase();
	  if (filtername.equals("pass")) {
	    String pass = filterenum.nextElement().toString();
	    if (DEBUG) System.out.println("filter:pass: "+pass);
	    passes = pass;
	    continue;
	  }
	  else if (filtername.equals("block")) {
	    String block = filterenum.nextElement().toString();
	    if (DEBUG) System.out.println("filter:block: "+block);
	    blocks = block;
	    continue;
	  }
	  else {
	    filterenum.nextElement();
	    if (DEBUG) System.out.println("filter: something else found");
	    continue;
	  }
	}
	continue;
      }
      else {
	if (DEBUG) System.out.println("Something else found");
	thelist.nextElement();
	continue;
      }
    }
  }

  /**
   * Puts a ratings description in this Profile.
   * @param state A String representing a ratings description.
   **/

  public void putState(String state) {

    states.addElement(state);
  }

	/**
   * Puts a full name (service/filename) into this Profile.
   * @param state A String representing a full name.
   **/

  public void putFullName(String fname) {

    fullnames.addElement(fname);
  }

	/**
	 * Returns the full names contained within this Profile.
	 * @return A Vector containing the full names (service/file) contained 
	 * within this Profile.
	 **/

	public Vector getFullNames() {
		return fullnames;
	}

  /**
   * Returns a String with the contents of the Profile.
   * @return A String representation of the Profile.
   **/

  public String getStringRep() {
    return stringrep;
  }

  /**
   * Returns the profiles contained within this Profile.
   * @return A Vector containing the names of the .pro files inside this 
   * profile.
   **/

  public Vector getPros() {
    return thepros;
  }

  /**
   * Returns the service descriptions contained within this Profile.
   * @return A Vector containing the names of the .rat files inside this 
   * profile.
   **/

  public Vector getRats() {
    return therats;
  }

  /**
	 * Returns the ratings settings contained within this Profile.
   * @return A Vector containing the saved state for each of the service 
   * descriptions within this profile.
   **/
  
  public Vector getStates() {
    return states;
  }

  /**
	 * Returns the Forbidden URLs contained within this Profile.
   * @return A Vector containing the Forbidden URLs for this Profile.
   **/
  
  public Vector getURLs() {
    return theurls;
  }

	/**
	 * Returns the Permitted URLs for contained within this Profile.
   * @return A Vector containing the Permitted URLs for this Profile.
   **/
  
  public Vector getURLs2() {
    return theurls2;
  }

  /**
	 * Returns the label sources contained within this Profile.
   * @return A Vector containing the label sources for this Profile.
   **/

  public Vector getLSources() {
    return lsources;
  }

	/**
	 * Returns the filenames contained within this Profile.
   * @return A Vector containing the filenames for this Profile.
   **/

  public Vector getFiles() {
    return thefiles;
  }

	/**
	 * Returns the booleans, in order, used in this Profile.
	 * @return A Vector containing the booleans (as Strings) for this Profile.
	 **/

	public Vector getBools() {
		return bools;
	}

	/**
	 * Returns the serviceinfo clauses (as LabelSources) contained within this 
	 * Profile.
	 * @return A Vector containing the LabelSource objects for this Profile.
	 * @see LabelSource
	 **/

	public Vector getServiceInfos() {
		return sinfos;
	}

	/**
   * Adds a filename to the internal table of filenames.
   * @param fname The filename to add.
   **/

  protected void addFile(String fname) {
    thefiles.addElement(fname);
  }

  /**
   * Adds a label source to the internal table of label sources.
   * @param sname The name of the label source.
   **/

  protected void addLSource(String sname) {
    lsources.addElement(sname);
  }

  /**
   * Adds a URL to the internal table of Forbidden URLs.  Checks to see
   * that it is in the proper format.
   * @param urlname The URL as a String.
   **/

  protected void addURL(String urlname) {
    try {
      URL testurl = new URL(urlname);
      theurls.addElement(urlname);
    }
    catch (MalformedURLException e) {
      System.out.println("Bad URL: "+urlname);
    }
  }

	/**
   * Adds a URL to the internal table of Permitted URLs.  Checks to see
   * that it is in the proper format.
   * @param urlname The URL as a String.
   **/

  protected void addURL2(String urlname) {
    try {
      URL testurl = new URL(urlname);
      theurls2.addElement(urlname);
    }
    catch (MalformedURLException e) {
      System.out.println("Bad URL: "+urlname);
    }
  }

  private void tokenize() {
    StringTokenizer stok = new StringTokenizer(stringrep, " ()");
    while (stok.hasMoreTokens()) {
      String thetok = stok.nextToken();
      if ((thetok.endsWith(".pro")) || (thetok.endsWith(".rulz"))) {
				thepros.addElement(thetok);
				continue;
			}
			if (thetok.equals("and")) {
				bools.addElement(thetok);
				continue;
			}
			/* if (thetok.equals("NOT"))
				continue; */
			if (thetok.equals("or")) {
				bools.addElement(thetok);
				continue;
			}
			therats.addElement(thetok);
    }
  }

	/**
	 * Returns the file name stored within this Profile (if any) 
	 * @return The name of this .pro file for this Profile.
	 **/

	public String getFilename() {
		return filename;
	}

	/** 
	 * Returns the directory name stored within this Profile (if any)
	 * @return The name of the directory for the .pro file for this Profile.
	 **/

	public String getDirname() {
		return dirname;
	}

	/**
	 * Returns the Pass portion of the Filter clause for this Profile.
	 * @return A String containing the Pass portion of the Filter.
	 **/

	public String getPass() {
		return passes;
	}

	/**
	 * Returns the Block portion of the Filter clause for this Profile.
	 * @return A String containing the Block portion of the Filter,
	 **/

	public String getBlock() {
		return blocks;
	}

	/**
	 * Returns the value of the description field of the name clause for this
	 * Profile.
	 * @return A String containing the description for this Profile.
	 **/

	public String getDesc() {
		return description;
	}

	/**
	 * Returns the value of the sourceURL field of the source clause for this 
	 * Profile.
	 * @return A String containing the sourceURL for this Profile.
	 **/

	public String getSourceURL() {
		return sourceURL;
	}

}
