// ModuleDatabase.java
// by Yang-hua Chu
// created: 11/2/96
// last update: 11/2/96

package w3c.pics.referee;

import java.io.* ;
import java.net.*;
import java.util.*;
import w3c.pics.util.*;

public class ModuleDatabase {

  String IDs[] = {};
  Object codeFragments[] = {};
  String interpreterIDs[] = {};  
  Vector supportedInterpreter = new Vector();
  
  public ModuleDatabase()
  {
    addSupportedInterpreter("http://www.javasoft.com/jdk/");
  }

  public int addPolicy(String ID, Object codeFragment, String interpreterID)
  {
    return addEntry(ID, codeFragment, interpreterID);
  }


  public int addInterpreter(String ID, RefereeInterpreter codeFragment, 
			    String interpreterID)
  {
    return addEntry(ID, codeFragment, interpreterID);
  }


  public int addInterpreter(String ID, String className, String interpreterID)
    throws RefereeException
  {
    try { 
      Class c = Class.forName(className); 
      RefereeInterpreter i = (RefereeInterpreter) c.newInstance();
      return addInterpreter(ID, i, interpreterID);
    
    }
    catch (Exception e) {throw new RefereeException(e.getMessage());}
  }


  private int addEntry(String ID, Object codeFragment, String interpreterID)
  {
    int index = lookup(ID);
    
    if (index != -1) {   // program name already exists, overridiing
      IDs[index] = ID;
      interpreterIDs[index] = interpreterID;
      codeFragments[index] = codeFragment;
      return index;
    }

    String nIDs[] = new String[size()+1];
    Object ncodeFragments[] = new Object[size()+1];
    String ninterpreterIDs[] = new String[size()+1];

    System.arraycopy(IDs, 0, nIDs, 0, size());
    System.arraycopy(codeFragments, 0, ncodeFragments, 0, size());
    System.arraycopy(interpreterIDs, 0, ninterpreterIDs, 0, size());

    nIDs[size()] = ID;
    ncodeFragments[size()] = codeFragment;
    ninterpreterIDs[size()] = interpreterID;

    IDs = nIDs;
    codeFragments = ncodeFragments;
    interpreterIDs = ninterpreterIDs;
    return size()-1;
  }


  public boolean removePolicy(String ID)
  {
    return removeEntry(ID);
  }


  public boolean removeInterpreter(String ID)
  {
    return removeEntry(ID);
  }


  public boolean removeEntry(String ID)
  {
    int index = lookup(ID);
    if (index == -1)
      return false;
    String nIDs[] = new String[size()-1];
    Object ncodeFragments[] = new Object[size()-1];
    String ninterpreterIDs[] = new String[size()-1];

    System.arraycopy(IDs, 0, nIDs, 0, index);
    System.arraycopy(codeFragments, 0, ncodeFragments, 0, index);
    System.arraycopy(interpreterIDs, 0, ninterpreterIDs, 0, index);
    System.arraycopy(IDs, index+1, nIDs, index, size()-index-1);
    System.arraycopy(codeFragments, index+1, ncodeFragments, index, 
		     size()-index-1);
    System.arraycopy(interpreterIDs, index+1, ninterpreterIDs, index, 
		     size()-index-1);

    IDs = nIDs;
    codeFragments = ncodeFragments;
    interpreterIDs = ninterpreterIDs;
    return true;
  }

  
  public boolean isInterpreterSupported(String name)
  {
    return supportedInterpreter.contains(name);
  }


  public void addSupportedInterpreter(String name)
  {
    supportedInterpreter.addElement(name);
  }


  public RefereeModule getModule(String actionName)
    throws RefereeException
  {
    int index = lookup(actionName);
    if (index == -1)
      throw new RefereeException("ModuleDatabase error: action name " +
				 actionName + " not found." +
				 this.toString());
    RefereeModule module = new RefereeModule();
    module.setPolicy(codeFragments[index]);

    while (! isInterpreterSupported(interpreterIDs[index])) {
      int newIndex = lookup(interpreterIDs[index]);
      if (newIndex == index) {
	System.out.println(this.toString() + actionName);
	throw new RefereeException("ModuleDatabase error: action name " + 
				   IDs[index] + " is interpreted by itself.");
      }
      if (newIndex == -1) {
	System.out.println(this.toString() + actionName);
	throw new RefereeException("ModuleDatabase error: action name " + 
				   IDs[index] + " not found.");
      }
      index = newIndex;
      module.addInterpreter((RefereeInterpreter)codeFragments[index]);
    }
    return module;
  }


  private int lookup(String name)
  {
    for (int i=0; i < size(); i++)
      if (IDs[i].equals(name))
	return i;
    return -1;
  }

  public String toString()
  {
    String s = "===== The Module Database contains =====\n";
    for (int i=0; i<size(); i++) {
      s += "Identifier  : " + IDs[i] + "\n";
      s += "CodeFragment: " + codeFragments[i] + "\n";
      s += "Interpreter : " + interpreterIDs[i] + "\n";
      s += "--------------------------------------\n";
    }
    return s;
  }

  public int size()
  {
    return IDs.length;
  }
}

