// PicsRULZInterpreter.java
// by Yang-hua Chu
// last update: 6/11/97

package w3c.pics.referee;

import java.io.* ;
import java.net.*;
import java.util.*;
import w3c.pics.util.*;
import w3c.pics.label.*;
import w3c.tools.sexpr.*;
import w3c.pics.profiler.*;


public class PicsRULZInterpreter implements RefereeInterpreter {
  
  private boolean DEBUG = false;

  public Hashtable curState = new Hashtable();
  
  public void initialize (Object o) 
  {
    return;
  }

  public void setPolicy(Object policy)
  {
    curState.put("POLICY", policy);
  }


  public void setModuleDatabase(ModuleDatabase moduleDB)
  {  
    curState.put("MODULE-DATABASE", moduleDB);
  }


  public ReturnedValues invoke(Vector v)
    throws RefereeException
  {
    curState.put("URL", v.elementAt(0));
    
    Profile rulz = (Profile) curState.get("POLICY");
    SExp exp = new SExp();
    exp.parseMultiple(SimpleSExprStream.toString(convert(rulz)));  
    if (DEBUG) System.out.println("the converted Profiles-0.92 policy is \n" + 
				  exp);
    ModuleDatabase moduleDB = (ModuleDatabase) curState.get("MODULE-DATABASE");

    RefereeModule module = new RefereeModule();
    module.setPolicy(exp);
    module.addInterpreter((RefereeInterpreter) moduleDB.getModule
			  ("http://www.w3.org/PICS/TrustMgt/Profiles092/").
			  getPolicy());
    module.setModuleDatabase(moduleDB);
    module.setPolicy(exp);
    Vector arg = new Vector();
    arg.addElement(curState.get("URL"));
    arg.addElement(new StatementList());
    ReturnedValues rval = module.invoke(arg);
    return rval;
  }


  public Vector convert(Profile rulz)
  {
    Vector out = new Vector();
    out.addElement(Symbol.makeSymbol("and", null));
    
    if (! rulz.getURLs().isEmpty()) {
      Vector failURL = new Vector();
      out.addElement(failURL);
      failURL.addElement(Symbol.makeSymbol("not", null));
      Vector urlMatch = new Vector();
      failURL.addElement(urlMatch);
      urlMatch.addElement(Symbol.makeSymbol("url-match", null));
      urlMatch.addElement(Symbol.makeSymbol("URL", null));
      urlMatch.addElement(rulz.getURLs());
    }

    Vector orClause = new Vector();
    out.addElement(orClause);
    orClause.addElement(Symbol.makeSymbol("or", null));

    if (! rulz.getURLs2().isEmpty()) {
      Vector passURL = new Vector();
      orClause.addElement(passURL);
      passURL.addElement(Symbol.makeSymbol("url-match", null));
      passURL.addElement(Symbol.makeSymbol("URL", null));
      passURL.addElement(rulz.getURLs2());
    }

    Vector letClause = new Vector();
    orClause.addElement(letClause);
    letClause.addElement(Symbol.makeSymbol("let", null));
    Vector bindClause = new Vector();
    letClause.addElement(bindClause);
    Vector filterClause = new Vector();
    letClause.addElement(filterClause);
    
    Enumeration sinfos = rulz.getServiceInfos().elements();
    while (sinfos.hasMoreElements()) {
      LabelSource sinfo = (LabelSource)sinfos.nextElement();
      Vector serviceClause = new Vector();
      bindClause.addElement(serviceClause);
      serviceClause.addElement(Symbol.makeSymbol(sinfo.getShortname(), null));
      Vector invoke = new Vector();
      serviceClause.addElement(invoke);
      invoke.addElement(Symbol.makeSymbol("invoke", null));
      invoke.addElement("load-label");
      invoke.addElement(Symbol.makeSymbol("STATEMENT-LIST", null));
      invoke.addElement(Symbol.makeSymbol("URL", null));
      invoke.addElement(sinfo.getName());
      Vector bureau = new Vector();
      invoke.addElement(bureau);
      bureau.addElement(sinfo.getBureauURL());
    }

    filterClause.addElement(Symbol.makeSymbol("and", null));
    filterClause.addElement(parseFilter(rulz.getPass()));

    Vector failFilter = new Vector();
    filterClause.addElement(failFilter);
    failFilter.addElement(Symbol.makeSymbol("not", null));
    failFilter.addElement(parseFilter(rulz.getBlock()));

    return out;
  }



  public Vector parseFilter(String s)
  {
    if (DEBUG) System.out.println(s);
    StringBufferInputStream ins = new StringBufferInputStream(s);
    SExprStream stream = new SimpleSExprStream(ins);    
    stream.setListsAsVectors(true);
    Vector v;
    try {
      v = (Vector) stream.parse();
    } catch (Exception e) { 
      System.err.println("parse error at PicsRulzReferee" + e.getMessage()); 
      return null; }
    return recurseParse(v);
  }
  
  private Vector recurseParse(Vector vin)
  {
    if (! (vin.elementAt(1).toString().equals("or") ||
	   vin.elementAt(1).toString().equals("and")))
      return chopFilter(vin);
    
    Vector vout = new Vector();
    vout.addElement(vin.elementAt(1));
    for (int i=0; i<vin.size(); i=i+2) {
      vout.addElement(recurseParse((Vector)vin.elementAt(i)));
    }
    return vout;
  }


  private Vector chopFilter(Vector vin)
  {
    String name = ((Symbol)vin.elementAt(0)).toString();
    String shortname = name.substring(0, name.indexOf('.'));
    String catagory = name.substring(name.indexOf('.')+1, name.length());
    String op = ((Symbol)vin.elementAt(1)).toString();
    String value = ((Integer)vin.elementAt(2)).toString();

    Vector match = new Vector();
    match.addElement(Symbol.makeSymbol("match", null));
    
    Vector pattern = new Vector();
    match.addElement(pattern);
    match.addElement(Symbol.makeSymbol(shortname, null));

    Vector speaker = new Vector();
    pattern.addElement(speaker);
    speaker.addElement("load-label");

    Vector assertion = new Vector();
    pattern.addElement(assertion);
    assertion.addElement(Symbol.makeSymbol("*", null));

    Vector label= new Vector();
    assertion.addElement(label);
    label.addElement(Symbol.makeSymbol("*", null));

    Vector ratings = new Vector();
    label.addElement(ratings);
    ratings.addElement(Symbol.makeSymbol("ratings", null));

    Vector restrict = new Vector();
    ratings.addElement(restrict);
    restrict.addElement(Symbol.makeSymbol("RESTRICT", null));
    restrict.addElement(Symbol.makeSymbol(op, null));
    restrict.addElement(Symbol.makeSymbol(catagory, null));
    restrict.addElement(Symbol.makeSymbol(value, null));
    return match;
  }    


  public static String readln(String msg, String def)
  {
    String s = "";
    System.out.print(msg + " [" + def + "] : ");
    System.out.flush();
    try { s = new DataInputStream(System.in).readLine(); }
    catch (Exception e) { }
    if (s.equals("")) s = def;
    return s;
  }    
  
  public static String toProfiles(String filename)
  {
    // input PicsRULZ
    Profile rulz = new Profile(new File(".", filename));

    // make conversion
    PicsRULZInterpreter module = new PicsRULZInterpreter();
    Vector v = module.convert(rulz);

    // output PicsRULZ
    SExp exp = new SExp(SimpleSExprStream.toString(v));
    return exp.toString();
  }


  public static void main(String[] argv)
       throws Exception
  {
    String progFileName;
    String labelFileName;
    String targetURLString;

    // 1st argument: PicsRULZ filename
    if (argv.length < 1) 
      progFileName = readln("Enter the filename for the Profile", "test-PicsRULZ/rulz1");
    else 
      progFileName = argv[0];
    
    // input PicsRULZ
    Profile rulz = new Profile(new File(".", progFileName));

    // make conversion
    PicsRULZInterpreter module = new PicsRULZInterpreter();
    Vector v = module.convert(rulz);

    // output PicsRULZ
    SExp exp = new SExp(SimpleSExprStream.toString(v));
    System.out.println(exp.toString());

  }
}

  

