// ProfilesInterpreter.java
// by Yang-hua Chu
// last update: 10/11/96

package w3c.pics.referee;

import java.io.* ;
import java.net.*;
import java.util.*;
import w3c.pics.util.*;
import w3c.pics.label.*;

public class ProfilesInterpreter implements RefereeInterpreter {
  
  public Hashtable curState = new Hashtable();
  
  // some of them may not be valid
  public boolean DEBUG=false;
  public boolean DEBUG_TERMINAL=false;
  public String PROGRAM_NAME = "Profiles";
  public SExp traceSExp = new SExp(SExp.SEXP_SEXP);
  public String endorsed_rater[] = null;
 

  public void initialize (Object o) 
  {
    return;
  }

  public void setPolicy(Object policy)
  {
    curState.put("POLICY", (SExp) policy);
  }


  public void setModuleDatabase(ModuleDatabase moduleDB)
  {  
    curState.put("MODULE-DATABASE", moduleDB);
  }


  public ReturnedValues invoke(Vector v)
    throws RefereeException
  {
    ((SExp)curState.get("POLICY")).reset();  // a bug in SExp

    curState.put("URL", (URL) v.elementAt(0));  
    curState.put("INITIAL-STATEMENT-LIST", (StatementList) v.elementAt(1)); 
    curState.put("STATEMENT-LIST", ((StatementList)v.elementAt(1)).clone());
    
    SExp program = (SExp) curState.get("POLICY");
    ReturnedValues rvalue = new ReturnedValues();

    for (int i=1; i <= program.size(); i++) 
      try {
	if (DEBUG) System.out.println("Line " + i + ": " + program.getSExp(i));
	rvalue = evalRule(program.getSExp(i), this.curState);
      } catch (Exception e) {
	e.printStackTrace();
	throw new RefereeException(e.getMessage());
      }
    return rvalue;
  }
  


  // rule::let-rule | explicit-rule | combine-rule | invoke-rule | 
  //       install-rule | projection-rule | match-rule | variable-name 
  public ReturnedValues evalRule(SExp exp, Hashtable curState) 
    throws ProfilesException
  {
    ReturnedValues rval = null;
    
    if (exp.getSExp(1).getString().equals("url-match"))
      rval = explicitRule(exp, curState);
    
    if (exp.getSExp(1).getString().equals("let"))
      rval = letRule(exp, curState);

    if (exp.getSExp(1).getString().equals("false-if-unknown") || 
	exp.getSExp(1).getString().equals("true-if-unknown") ||
	exp.getSExp(1).getString().equals("not")) 
      rval = unaryCombine(exp, curState);
    
    if (exp.getSExp(1).getString().equals("or"))
      rval = combineRuleOr(exp, curState);
    
    if (exp.getSExp(1).getString().equals("and"))
      rval = combineRuleAnd(exp, curState);
    
    if (exp.getSExp(1).getString().equals("invoke"))
      rval = invokeRule(exp, curState);
        
    if (exp.getSExp(1).getString().equals("match"))
      rval = matchRule(exp, curState);
    
    // the rules below are not implemented yet
    //     if ((exp.expect(exp.SEXP_IDENT, "install-program")) ||
    // 	(exp.expect(exp.SEXP_IDENT, "install-interpreter")))
    //       return installRule(exp, exp.getString());
    
    //     if ((exp.expect(exp.SEXP_IDENT, "tri-value")) ||
    // 	(exp.expect(exp.SEXP_IDENT, "statement-list")))
    //       return projectionRule(exp, exp.getString());
    
    if (rval == null)
      throw new ProfilesException("unknown rule " + exp.toStringShort());
    
    if (DEBUG) System.out.println("Eval:\n" + exp.toString() + "\n");
    if (DEBUG_TERMINAL) halt();

    ((StatementList)curState.get("STATEMENT-LIST")).merge(rval.getStatementList());
    if (DEBUG) System.out.println("Result:\n" + rval.toString() + "\n");
    if (DEBUG_TERMINAL) halt();

    traceSExp.addSExp(exp);
    traceSExp.addSExp(new SExp(rval.toString()));
    return rval;
  }


  // (false-if-unknown rule)
  // (true-if-unknown rule)
  // (not rule)
  private ReturnedValues unaryCombine(SExp exp, Hashtable curState)
    throws ProfilesException
  {
    ReturnedValues rval = evalRule(exp.getSExp(2), curState);

    if (exp.getSExp(1).getString().equals("not")) 
      rval.setTriValue(rval.getTriValue().not());
    
    if (exp.getSExp(1).getString().equals("false-if-unknown")) 
      rval.setTriValue(rval.getTriValue().falseIfUnknown());
      
    if (exp.getSExp(1).getString().equals("true-if-unknown")) 
      rval.setTriValue(rval.getTriValue().trueIfUnknown());

    return rval;
  }
  

  // if true, return all statements that are true
  // if false, return all statements that are false
  private ReturnedValues matchRule(SExp exp, Hashtable curState)
    throws ProfilesException
  {
    ReturnedValues rval = new ReturnedValues();
    StatementList true_stmt_list = new StatementList();
    StatementList false_stmt_list = new StatementList();
    StatementList restrict_stmt_list = new StatementList();
    SExp pattern = new SExp();
    
    StatementList stmtlist = (StatementList)
      curState.get(exp.getSExp(3).toString());

    if ( exp.getSExp(2).ttype != exp.SEXP_SEXP)
      throw new ProfilesException("expect a valid pattern" + 
				  exp.getSExp(2).toString());
    pattern.parse(exp.getSExp(2).toString());

    for (int i=1; i <= stmtlist.size(); i++) {
      try { 
	if (stmtlist.getStatement(i).restrictMatch(pattern))
	  true_stmt_list.addStatement(stmtlist.getStatement(i));
	else
	  false_stmt_list.addStatement(stmtlist.getStatement(i));
      } catch (SExpException e) {
	if (e.getMessage().equals("has !"))
	  restrict_stmt_list.addStatement(stmtlist.getStatement(i));
	if (e.getMessage().equals("no !")) 
	  false_stmt_list.addStatement(stmtlist.getStatement(i));
      }
    }
    
    if (restrict_stmt_list.size() > 0) {
      rval.setTriValue(new TriValue (false));
      rval.setStatementList(restrict_stmt_list);
      return rval;
    }
    if (true_stmt_list.size() > 0) {
      rval.setTriValue(new TriValue (true));
      rval.setStatementList(true_stmt_list);
      return rval;
    }
    rval.setTriValue(new TriValue (false));
    rval.setStatementList(false_stmt_list);
    return rval;
  }


  // (let (<bindings>+) <rule>+)
  private ReturnedValues letRule(SExp exp, Hashtable curState)
    throws ProfilesException
  {
    ReturnedValues rval = new ReturnedValues();
    
    SExp bindings = exp.getSExp(2);
    Hashtable newState = (Hashtable)curState.clone();

    for (int i=1; i<=bindings.size(); i++) {
      Hashtable tmpState = (Hashtable)curState.clone();
      SExp bind = bindings.getSExp(i);
      String name = bind.getSExp(1).toString();
      rval = evalRule(bind.getSExp(2), tmpState);
      newState.put(name, rval.getStatementList());
    }
    
    for (int i=3; i<=exp.size(); i++) {
      rval = evalRule(exp.getSExp(i), newState);
    }
    return rval;
  }
  

  // (and rule1 rule2 rule3)
  private ReturnedValues combineRuleAnd(SExp exp, Hashtable curState)
    throws ProfilesException
  {
    ReturnedValues rval = new ReturnedValues();
    rval.setTriValue(new TriValue (true));
    for (int i=2; i<=exp.size(); i++) {
      ReturnedValues nrval = evalRule(exp.getSExp(i), curState);
      // if any operant returns false, the rule returns without further
      // evaluation
      if (nrval.getTriValue().equals(new TriValue(false)))
	return nrval;
      if (nrval.getTriValue().equals(rval.getTriValue())) {
	rval.getStatementList().merge(nrval.getStatementList());
	continue;
      }
      rval.setTriValue(rval.getTriValue().and(nrval.getTriValue()));
      if (nrval.getTriValue().equals(rval.getTriValue()))
	rval.setStatementList(nrval.getStatementList());
    }
    return rval;
  }
  

  // (or rule1 rule2 rule3)
  private ReturnedValues combineRuleOr(SExp exp, Hashtable curState)
    throws ProfilesException
  {
    ReturnedValues rval = new ReturnedValues();    
    rval.setTriValue(new TriValue (false));
    for (int i=2; i<=exp.size(); i++) {
      ReturnedValues nrval = evalRule(exp.getSExp(i), curState);
      // if any operant returns true, the rule returns without further
      // evaluation
      if (nrval.getTriValue().equals(new TriValue(true)))
	return nrval;
      if (nrval.getTriValue().equals(rval.getTriValue())) {
	rval.getStatementList().merge(nrval.getStatementList());
	continue;
      }
      rval.setTriValue(rval.getTriValue().or(nrval.getTriValue()));
      if (nrval.getTriValue().equals(rval.getTriValue()))
	rval.setStatementList(nrval.getStatementList());
    }
    return rval;
  }
  

  // now just return a default PICS label
  // example:
  // (invoke "load-label" STATEMENT-LIST URL "http://www.musac.org/"
  //         (EMBEDDED))
  // right now STATEMENT-LIST and URL variables are not used.
  private ReturnedValues invokeRule(SExp exp, Hashtable curState)
    throws ProfilesException
  {
    ReturnedValues rval = new ReturnedValues();
    rval.setTriValue(new TriValue (false));

    // get the REFEREE module
    String actionName = exp.getSExp(2).getString();
    ModuleDatabase pgdb = (ModuleDatabase)curState.get("MODULE-DATABASE");
    RefereeModule module;
    try { module = pgdb.getModule(actionName);} 
    catch (Exception e) {
      e.printStackTrace();
      throw new ProfilesException(e.getMessage());
    }
    module.setModuleDatabase(pgdb);
    Vector arg = new Vector();
    
    // construct arguments
    for (int i=3; i<=exp.size(); i++) {
      if (exp.getSExp(i).ttype == SExp.SEXP_IDENT) {
	Object o = curState.get(exp.getSExp(i).getString());
	if (o != null) {
	  arg.addElement(o);
	  continue;
	}
      }
      arg.addElement(exp.getSExp(i).toString());
    }

    // runs the module
    try {rval = module.invoke(arg); }
    catch (Exception e) {
      e.printStackTrace();
      throw new ProfilesException(e.getMessage());}

    // prepend the module name in the context of the statements returned
    rval.getStatementList().prependContext(actionName);
    return rval;
  }
  

  // ( url-match  ) => return true
  // (url-match URL [url]+ <?prefix>)
  private ReturnedValues explicitRule(SExp exp, Hashtable curState)
       throws ProfilesException
  {
    ReturnedValues rvals = new ReturnedValues();

    // if action follows no argument, return a TriValue and an empty stmtList
    if (exp.size() == 1) {
      rvals.setTriValue(new TriValue('t'));
      return rvals;
    }

    // handle the more complicated case, url-match with URLs
    boolean matched = false;
    URL matchingURL;
    SExp matchList = null;
    
    if ((exp.getSExp(2).ttype != exp.SEXP_IDENT) ||
	(! exp.getSExp(2).getString().equals("URL")))
      throw new ProfilesException("expect keyward URL");
    
    if (exp.getSExp(3).ttype != exp.SEXP_SEXP)
      throw new ProfilesException("expect a list of matching URLs");
    matchList = exp.getSExp(3);

    for (int index=1; index <= matchList.size(); index++) {
      if (matchList.getSExp(index).ttype != SExp.SEXP_STRING)
	throw new ProfilesException(matchList.getSExp(index).toString() +
				    " is not an URL");      
      try {
	matchingURL = new URL(matchList.getSExp(index).getString());
      } catch (MalformedURLException e) {
	throw new ProfilesException("Unable to parse the URL" + 
				    matchList.getSExp(index).toString());
      }
      
      if (((URL)curState.get("URL")).toString().startsWith
				     (matchingURL.toString())) {
	Statement newStmt = new Statement();
	newStmt.prependContext(PROGRAM_NAME);
	newStmt.getContent().addSExp(new SExp(newStmt.SEXP_STRING,"url-match"));
	newStmt.getContent().addSExp(new SExp(newStmt.SEXP_STRING, 
				     matchingURL.toString()));
	rvals.getStatementList().addStatement(newStmt);
      }
    }
    if (rvals.getStatementList().size() == 0)
      rvals.setTriValue(new TriValue('f'));
    else
      rvals.setTriValue(new TriValue('t'));
    
    return rvals;
  }


  public static void halt()
  {
      System.out.print("Press return to continue > ");
      System.out.flush();
      try { int j = System.in.read(); }
      catch (Exception e) { }
      System.out.print("\n\n");
  }

  public static String readln(String msg, String def)
  {
    String s = "";
    System.out.print(msg + " [" + def + "] : ");
    System.out.flush();
    try { s = new DataInputStream(System.in).readLine(); }
    catch (Exception e) { }
    if (s.equals("")) s = def;
    return s;
  }    
  
} // class Profiles

  





