package w3c.pics.referee;

import java.util.*;
import java.net.*;
import java.io.*;

import w3c.pics.util.*;
import w3c.www.protocol.http.*;
import w3c.jigsaw.forms.*;
import w3c.jigsaw.html.*;
import w3c.www.http.*;
import w3c.pics.profiler.*;

/**
 * A simple debug filter, that will dump traffic
 * This filter will dump the outgoing request, and the incomming reply to
 * the java process standard output.
 * <p>Usefull for debugging !
 */

public class RefereeFilter implements PropRequestFilter {

  private boolean DEBUG=false;
  private static ModuleDatabase moduleDB = new ModuleDatabase();
  private static StatementList initStmtList = new StatementList();
  
  static {
    try {
      SExp tmp = new SExp();
      tmp.parseMultiple(new FileInputStream("ProfilesPolicy"));
      moduleDB.addPolicy("view", tmp, 
			 "http://www.w3.org/PICS/TrustMgt/Profiles092/");
    } catch (Exception e4) {
      System.err.println("bad policy or policy cannot be found");
    }

    try {
      moduleDB.addInterpreter("http://www.w3.org/PICS/TrustMgt/Profiles092/",
			      "w3c.pics.referee.ProfilesInterpreter",
			      "http://www.javasoft.com/jdk/");
    } catch (Exception e) {
      System.err.println("w3c.pics.referee.ProfilesInterpreter " + 
			 "cannot be found");
    }

    try {
      SExp tmp = new SExp();
      tmp.parseMultiple(new FileInputStream("PICSLabel"));
      moduleDB.addPolicy("load-label", tmp, 
			 "http://www.w3.org/PICS/TrustMgt/LabelLoader/");
    } catch (Exception e4) {
      System.err.println("bad label file or label file cannot be found");
    }

    try {
      moduleDB.addInterpreter("http://www.w3.org/PICS/TrustMgt/LabelLoader/", 
			      "w3c.pics.referee.LabelLoaderInterpreter", 
			      "http://www.javasoft.com/jdk/");
    } catch (Exception e) {
      System.err.println("w3c.pics.referee.LabelLoaderInterpreter" + 
			 " cannot be found");
    }

    try {
      Profile rulz = new Profile(new File(".", "rulz"));
      moduleDB.addPolicy("view-RULZ", rulz, 
			 "http://www.w3.org/PICS/TrustMgt/PicsRULZ/");
    } catch (Exception e4) {
      System.err.println("bad PicsRULZ file");
    }
    try {
      moduleDB.addInterpreter("http://www.w3.org/PICS/TrustMgt/PicsRULZ/", 
			      "w3c.pics.referee.PicsRULZInterpreter", 
			      "http://www.javasoft.com/jdk/");
    } catch (Exception e) {
      System.err.println("w3c.pics.referee.PicsRULZInterpreter" +
			 " cannot be found");
    }

    try {
      moduleDB.addPolicy("load-URL", null, 
			 "http://www.w3.org/PICS/TrustMgt/URLLoader/");
    } catch (Exception e4) {
      System.err.println("bad URL Fetcher");
    }

    try {
      moduleDB.addInterpreter("http://www.w3.org/PICS/TrustMgt/URLLoader/", 
			      "w3c.pics.referee.URLLoader", 
			      "http://www.javasoft.com/jdk/");
    } catch (Exception e) {
      System.err.println("w3c.pics.referee.URLLoader" +
			 " cannot be found");
    }
    System.out.println(moduleDB.toString());
  }


  /**
   * This filter doesn't handle exceptions.
   * @param request The request that triggered the exception.
   * @param ex The triggered exception.
   * @return Always <strong>false</strong>.
   */
  
  public boolean exceptionFilter(Request request, HttpException ex) {
    return false;
  }

  /**
   * PropRequestFilter implementation - Initialize the filter.
   * Time to register ourself to the HttpManager.
   * @param manager The HTTP manager that is initializing ourself.
   */
  
  public void initialize(HttpManager manager) {
    // We install ourself as a global filter, we are cool !
    manager.setFilter(this);
  }
  
  
  public String toString()  {
    String s = "";
    s += "moduleDB is " + moduleDB.toString();
    s += "initStmtList is " + initStmtList.toString();
    return s;
  }


  /**
   * The ingoing filter just dumps the request.
   * @param request The request to be filtered.
   * @exception HttpException is never thrown.
   */
  public Reply ingoingFilter(Request request) 
    throws HttpException
  {
    System.out.println(request.getManager().getGlobalHeader("User-Agent"));
    
    if ((request.getManager().getGlobalHeader("TrustMgt-Flag") != null) &&
	(request.getManager().getGlobalHeader("TrustMgt-Flag").equals("false")))
	{
      request.getManager().setGlobalHeader("TrustMgt-Flag", "true");
      return null;
    }

    System.out.println(" ------------- Begin REFEREE Filtering ---------");
    // binds a request to an action name
    URL requestedURL = request.getURL();
    System.out.println("Requested URL = \"" + requestedURL + "\"");
    String requestString = requestedURL.toString();
    int index = requestString.lastIndexOf(".")+1;
    int indexSlash = requestString.lastIndexOf("/");
    if (index < indexSlash)
      index = indexSlash;
    String fileType = requestString.substring(index);
    System.out.println("The File Type = \"" + fileType + "\"");
    String action = "";   
    String trueAction = "http://www.w3.org/PICS/TrustMgt/release/970616-true.html";
    String unknownAction = "http://www.w3.org/PICS/TrustMgt/release/970616-unknown.html";
    String falseAction = "http://www.w3.org/PICS/TrustMgt/release/970616-false.html";
    
    String defaultAction = "view";
    if (requestedURL.toString().endsWith("html") ||
	requestedURL.toString().endsWith("HTML") ||
	requestedURL.toString().endsWith("htm") ||
	requestedURL.toString().endsWith("HTM"))
      action = "view";
    if (requestedURL.toString().endsWith("gif") ||
	requestedURL.toString().endsWith("GIF") ||
	requestedURL.toString().endsWith("jpg") ||
	requestedURL.toString().endsWith("JPG")) {
      action = "view-RULZ";
      unknownAction = "http://www.w3.org/PICS/TrustMgt/release/970616-warning.gif";
      falseAction = "http://www.w3.org/PICS/TrustMgt/release/970616-warning.gif";
    }
    //if (requestedURL.toString().endsWith("java"))
    //action = "applet";
    
    if (action.equals("")) {
      action = defaultAction;
    }
    System.out.println("The file suffix " + fileType + " binds to the " +
			 "action name " +defaultAction);
      
    RefereeModule module;
    try {
      module = moduleDB.getModule(action);
    } catch (RefereeException e) {
      System.err.println("RefereeFilter error: action name " + action +
			 " not found in the following REFEREE Module " +
			 "Database:\n" + moduleDB);
      throw new HttpException(request, e.getMessage());
    }
    module.setModuleDatabase(moduleDB);
    Vector arg = new Vector();
    arg.addElement(requestedURL);
    arg.addElement(initStmtList);
    ReturnedValues rval;
    try {
      rval = module.invoke(arg);
    } catch (RefereeException e) {
      System.err.println("RefereeFilter error: error when invoking action " +
			 "actioin.\n");
      toString();
      e.printStackTrace();
      throw new HttpException(request, e.getMessage());
    }

    System.out.println("result of " + requestedURL.toString());
    System.out.println(rval);



    URL actionURL;
    String actionString = "";
    if (rval.getTriValue().equals(new TriValue('t'))) 
      return null;
    if (rval.getTriValue().equals(new TriValue('u')))
      actionString = unknownAction;
    else
      actionString = falseAction;

    try {
      actionURL = new URL(actionString);
    } catch (Exception e) {
      System.err.println("Initialization failed: URL cannot be parsed");
      throw new HttpException(request, e.getMessage());
    }
    
    try {
      module = moduleDB.getModule("load-URL");
    } catch (RefereeException e) {
      System.err.println("RefereeFilter error: action name \"load-URL\"" +
			 " not found in the following REFEREE Module " +
			 "Database:\n" + moduleDB);
      throw new HttpException(request, e.getMessage());
    }
    module.setModuleDatabase(moduleDB);
    Vector arg2 = new Vector();
    arg2.addElement(actionURL);
    arg2.addElement(this);
    try {
      rval = module.invoke(arg2);
    } catch (RefereeException e) {
      System.err.println("RefereeFilter error: error when invoking action " +
			 "actioin.\n");
      toString();
      e.printStackTrace();
      throw new HttpException(request, e.getMessage());
    }
    return (Reply)rval.getStatementList().getStatement(1).getContent().
      getSExp(1).getObject();
  }
  
  /**
   * The outgoing filter just dumps the reply.
   * @param request The request that is filtered.
   * @param reply The corresponding reply.
   * @exception HttpException is never thrown.
   */
  
  public Reply outgoingFilter(Request Request, Reply reply) 
	throws HttpException
  {
    // On the way out, emit the reply to stdou:
    reply.dump(System.out);
    return null;
  }
  
    /**
     * We do not maintain any in-memory cached state.
     */
  
  public void sync() {
    return;
  }

}




