// SExpTokenizer.java
// by Yang-hua Chu
// last update: 10/11/96

package w3c.pics.util;

import java.io.* ;
import java.net.*;
import java.util.*;

public class SExp {

  private final boolean DEBUG=false;

  public final static int SEXP_STRING = 1;
  public final static int SEXP_IDENT = 2;
  public final static int SEXP_NUMBER = 3;
  public final static int SEXP_SEXP = 4;
  public final static int SEXP_OBJECT = 5;

  private static int UNINITIALIZED = 0;

  public SExp sexps[] = {};
  public int lineno = 0;
  public int ttype = UNINITIALIZED;
  public double nval = 0;
  public String sval = "";
  public Object oval;
  private int ptr = -1;

  public Object clone()
  {
    SExp exp = new SExp();
    exp.lineno = this.lineno;
    exp.ttype = this.ttype;
    exp.nval = this.nval;
    exp.sval = this.sval;
    exp.ptr = this.ptr;
    if (this.ttype == SEXP_SEXP)
      for (int i=1; i <= this.size(); i++)
	exp.addSExp((SExp) this.getSExp(i).clone());
   return exp;
  }

  public SExp()
  {
  }

  public SExp(int type, String str)
  {
    ttype = type;
    sval = str;
  }

  public SExp(int type, Object o)
  {
    ttype = type;
    oval = o;
  }

  public SExp(int type, double val)
  {
    ttype = type;
    nval = val;
  }

  public SExp(int type)
  {
    ttype = type;
  }

  public SExp(String s)
  {
    parse(s);
  }

  public boolean parse(String s)
  {
    StringBufferInputStream in = new StringBufferInputStream(s);
    return parse(in);
  }

  // return true if parse is sucessful, false otherwise;
  public boolean parse(InputStream in)
  {
    StreamTokenizer ins = new StreamTokenizer(in);
    ins.wordChars('<', '<');
    ins.wordChars('>', '>');
    ins.wordChars('=', '=');
    ins.wordChars('*', '*');
    ins.wordChars('+', '+');
    ins.wordChars('!', '!');
    ins.wordChars('?', '?');
    ins.wordChars(':', ':');// for multi-value PICS rating such as ( 2 : 3 )
    try { this.parseRecursive(ins); } 
    catch (SExpException e) { return false; }
    return true;
  }
  
  // return true if parse is sucessful, false otherwise;
  public boolean parseMultiple(String s)
  {
    StringBufferInputStream in = new StringBufferInputStream(s);
    return parseMultiple(in);
  }

  public boolean parseMultiple(InputStream in)
  {
    StreamTokenizer ins = new StreamTokenizer(in);
    ins.wordChars('<', '<');
    ins.wordChars('>', '>');
    ins.wordChars('=', '=');
    ins.wordChars('*', '*');
    ins.wordChars('+', '+');
    ins.wordChars('!', '!');
    ins.wordChars('?', '?');
    try {
      while (true) {
	SExp exp = new SExp();
	exp.parseRecursive(ins);
	this.addSExp(exp);
	this.ttype = SEXP_SEXP; } 
    } catch (SExpException e) {
	return (this.size() != 0);
    }
  }
  
  private void parseRecursive(StreamTokenizer ins)
       throws SExpException
  {
    try {
      int tokenType = ins.nextToken();
      this.lineno = ins.lineno();
      
      if (DEBUG) System.out.println(ins.toString());
      
      switch (tokenType) {
      case ins.TT_NUMBER:
	this.ttype = SEXP_NUMBER;
	this.nval = ins.nval;
	return;
      case ins.TT_WORD: // identifier
	this.ttype = SEXP_IDENT;
	this.sval = ins.sval;
	return;
      case 34: // string '"'?
	this.ttype = SEXP_STRING;
	this.sval = ins.sval;
	return;
      case '(':
	this.ttype = SEXP_SEXP;
	while (ins.nextToken() != ')') {
	  ins.pushBack();
	  SExp exp = new SExp();
	  exp.parseRecursive(ins);
	  this.addSExp(exp);
	}
	if (DEBUG) System.out.println(ins.toString());
	return;
      case ')':
	throw new SExpException("unexpected ')' at line " + ins.lineno());
      case ins.TT_EOF:
	throw new SExpException("EOF reached at line " + ins.lineno());
      default:
	throw new SExpException("unseen token type "+ins.toString());
      } 
    } catch (IOException e) {
      throw new SExpException(e.getMessage());
    }
  }
    
  public void addSExp(SExp exp)
  {
    SExp nsexps[] = new SExp[sexps.length + 1] ;
    System.arraycopy (sexps, 0, nsexps, 0, sexps.length) ;
    sexps = nsexps;
    sexps[sexps.length-1] = exp;
  }

  public void prependSExp(SExp exp)
  {
    SExp nsexps[] = new SExp[sexps.length + 1] ;
    System.arraycopy (sexps, 0, nsexps, 1, sexps.length) ;
    sexps = nsexps;
    sexps[0] = exp;
  }

  public void addIdent(String s)
  {
    addSExp(new SExp(SEXP_IDENT, s));
  }

  public void addString(String s)
  {
    addSExp(new SExp(SEXP_STRING, s));
  }

  public void addNumber(double d)
  {
    addSExp(new SExp(SEXP_NUMBER, d));
  }

  public void addObject(Object o)
  {
    addSExp(new SExp(SEXP_OBJECT, o));
  }

  public void setSExp(int index, SExp exp)
  {
    if (index > sexps.length) {      
      SExp nsexps[] = new SExp[index] ;
      System.arraycopy (sexps, 0, nsexps, 0, sexps.length) ;
      sexps = nsexps;      
    }
    sexps[index-1] = exp;
  }

  // size is valid on only if the expression is SEX_SEXP
  public int size()
  {
    if (ttype != SEXP_SEXP)
      return 0;
    return sexps.length;
  }

  public boolean match(SExp exp)
  {
    try {
      return restrictMatch(exp);
    } catch (SExpException e) {
      return false;
    }
  }
  
  public boolean restrictMatch(SExp exp)
    throws SExpException
  {
    SExp nexp;
    SExp texp;
    if ((exp.ttype != SEXP_SEXP) ||
	(this.ttype != SEXP_SEXP)) {
      nexp = new SExp(SEXP_SEXP);
      texp = new SExp(SEXP_SEXP);
      nexp.addSExp((SExp) exp.clone());
      texp.addSExp((SExp) this.clone());
    } else {
      nexp = ((SExp) exp.clone());
      texp = ((SExp) this.clone());
    }
      return texp.matchRecursive(nexp);
  }
  
  private boolean compareRestriction(SExp exp)
    throws SExpException
  {
    String op = exp.current().toString();
    try {
      this.nextToken();
      exp.nextToken();
      String transmitName = this.current().getString();
      String matchTransmitName = exp.current().getString();
      if (! transmitName.equals(matchTransmitName))   // wrong transmit name
	return false;
      
      this.nextToken();
      exp.nextToken();
    } catch (SExpException e) {
      return false;
    }

    double num1 = this.current().getNumber();
    double num2 = exp.current().getNumber();
    
    if (op.equals("<"))
      if (num1 < num2)	return true;
      else throw new SExpException ("no !");
    
    if (op.equals(">"))
      if (num1 > num2)	return true;
      else throw new SExpException ("no !");
    
    if (op.equals("="))
      if (num1 == num2)	return true;
      else throw new SExpException ("no !");
    
    if (op.equals("<="))
      if (num1 <= num2)	return true;
      else throw new SExpException ("no !");
    
    if (op.equals(">="))
      if (num1 >= num2)	return true;
      else throw new SExpException ("no !");
    
    if (op.equals("<!"))
      if (num1 < num2)	return true;
      else throw new SExpException ("has !");
    
    if (op.equals(">!"))
      if (num1 > num2)	return true;
      else throw new SExpException ("has !");
    
    if (op.equals("=!"))
      if (num1 == num2)	return true;
      else throw new SExpException ("has !");
    
    if (op.equals("<=!"))
      if (num1 <= num2)	return true;
      else throw new SExpException ("has !");
    
    if (op.equals(">=!"))
      if (num1 >= num2)	return true;
      else throw new SExpException ("has !");
    
    return false;
  }

  // exp is the match description file
  private boolean matchRecursive(SExp exp)
    throws SExpException
  {
    // at the beginning exp forward once
    while (true) {
      try { exp.nextToken(); }
      catch (Exception e) { 
	try { this.nextToken(); }
	catch (Exception e2) { return true; }
	return false;
      }
      
      switch (exp.current().ttype) {

      case SEXP_IDENT:
	// special case: '+', '*': zero or more token
	if ( exp.current().getString().equals("*") || 
	     exp.current().getString().equals("+") ) {

	  // '+' needs to forward the token once.
	  if (exp.current().getString().equals("+")) {
	    try {this.nextToken(); }
	    catch (Exception e) {return false;} 
	  }
	  try { exp.nextToken(); }   // more comparision in SExp matcher?
	  catch (Exception e) { return true; }  // no, match found
	  finally { exp.pushBack(); }

	  while (true) {
	    if (((SExp)this.clone()).matchRecursive((SExp) exp.clone()))
	      return true;
	    try { this.nextToken(); }  // more tokens in SExp matcher?
	    catch (Exception e) { return false; }
	  }
	}
	
	// special case '?': zero or one token
	if (exp.current().getString().equals("?")) {
	  try { exp.nextToken(); }   // more comparision in SExp matcher?
	  catch (Exception e) {      // if not
	    try { this.nextToken(); this.nextToken(); }  // no more token
	    catch (Exception e2) {return true; }  // winner
	    return false;                        // not a match
	  }
	  finally { exp.pushBack(); }
	  
	  if (((SExp) this.clone()).matchRecursive((SExp) exp.clone()))
	    return true;
	  
	  try { this.nextToken(); }  // more tokens in SExp matcher?
	  catch (Exception e) { return false; }
	  if (((SExp) this.clone()).matchRecursive((SExp) exp.clone()))
	    return true;
	  return false;
	}
	
	// speical case: restriction operators
	// restriction operators are valid only if it is in the first 
	// element of an s-expression
	if (exp.current().getString().equals("RESTRICT") && (exp.ptr == 0)) {
	  try {exp.nextToken(); }
	  catch (Exception e2) {return false;}
	  if ((exp.ptr == 1) && ((exp.current().getString().equals(">")) ||
				 (exp.current().getString().equals("<")) ||
				 (exp.current().getString().equals("=")) ||
				 (exp.current().getString().equals("<>")) ||
				 (exp.current().getString().equals(">=")) ||
				 (exp.current().getString().equals("<=")) ||
				 (exp.current().getString().equals("<!")) ||
				 (exp.current().getString().equals(">!")) ||
				 (exp.current().getString().equals("=!")) ||
				 (exp.current().getString().equals(">=!")) ||
				 (exp.current().getString().equals("<=!")) ||
				 (exp.current().getString().equals("<>!")))) {
	    return compareRestriction(exp);
	  }
	  else exp.pushBack();
	}
	
	// normal cases for SEXP_IDENT and SEXP_STRING
      case SEXP_STRING:
	try { this.nextToken(); }  // more tokens in SExp matcher?
	catch ( Exception e) { return false; }
	if ((current().ttype == exp.current().ttype) &&
	    (current().sval.equals(exp.current().sval))) 
	  continue;
	return false;
      
      case SEXP_NUMBER:
	try { this.nextToken(); }  // more tokens in SExp matcher?
	catch (Exception e) { return false; }
	if ((current().ttype == exp.current().ttype) &&
	    (exp.current().nval == current().nval)) 
	  continue;
	return false;
	
      case SEXP_SEXP:
	try { this.nextToken(); }  // more tokens in SExp matcher?
	catch ( Exception e) { return false; }
	if ((current().ttype == exp.current().ttype) && 
	    (current().matchRecursive(exp.current())))
	  continue;
	return false;
      default:
	System.out.println("ERROR!!!");
	return false;
      }
    }
  }


  public String toString()
  {
    return toString("");
  }
  
  public String toString(String indent)
  {
    return toString(indent, false);
  }
  
  private String toString(String indent, boolean isIndent)
  {
    switch (ttype) {
    case SEXP_NUMBER: return ("" + nval);
    case SEXP_STRING: return ("\"" +  sval + "\"");
    case SEXP_IDENT: 
      if (isIndent && (sval.equals("at") || sval.equals("by") || 
		       sval.equals("comment") || sval.equals("exp") || 
		       sval.equals("until") || sval.equals("extension") || 
		       sval.equals("for") || sval.equals("full") || 
		       sval.equals("complete-label") || sval.equals("gen") || 
		       sval.equals("generic") || sval.equals("on") || 
		       sval.equals("labels") || sval.equals("md5") ||
		       sval.equals("ratings")))
	return ("\n" + indent + sval);
      return sval;
    case SEXP_OBJECT:
      return ("<" + oval.getClass().getName() + ">");
    case SEXP_SEXP:
     String s = "";
     if (isIndent) s += "\n" + indent;
     s += "(";
     for (int i=0; i< size(); i++) {
       s += " " + sexps[i].toString(indent + "  ", (i!=0));
       // when encountered 'label' word, indent further
       if ((sexps[i].ttype == SEXP_IDENT) && 
	   sexps[i].getString().equals("labels"))
	 indent += "  ";
     }
     s += " )";
     return s;
   }
   return "";
  }


  public String toStringShort()
 {
   String s = "";
   switch (ttype) {
   case SEXP_NUMBER: s += nval; break;
   case SEXP_IDENT: s += sval; break;
   case SEXP_STRING: s += "\"" +  sval + "\""; break;
    case SEXP_OBJECT: s += "<" + oval.getClass().getName() + ">";
   case SEXP_SEXP:
     s += "(";
     for (int i=0; i< this.size(); i++)
       if (this.sexps[i].ttype == SEXP_SEXP)
	 s += " (...)";
       else
	 s += " " + this.sexps[i].toStringShort();
     s += " )";
     break;
   }
   return s;
 }
  

  // simulate a TokenStream
  public void reset()
  {
    ptr = -1;
    for (int i=1; i<=size(); i++)
      if (getSExp(i).ttype == SEXP_SEXP) 
	getSExp(i).reset();
  }

  public boolean isLast()
  {
    if (ptr >= size())
      return false;
    else
      return true;
  }

  public void pushBack()
  {
    ptr--;
  }
  
  public int nextToken()
       throws SExpException
  {
    if ((ptr+1) >= size())
      throw new SExpException("no more sub-expression");
    ptr++;
    return sexps[ptr].ttype;
  }

  public SExp current()
  {
      return sexps[ptr];
  }

  public boolean expect(int tokenType)
  {
    int t;
    
    if ((ptr+1) >= size())
      return false;
    if (DEBUG) System.out.println(this.sexps[ptr+1].toStringShort());
    if (this.sexps[ptr+1].ttype == tokenType) {
      ptr++;
      return true;
    } else
      return false;
  }
  
  public boolean expect(int tokenType, String s)
  {
    if (ptr >= size())
      return false;

    if (DEBUG) System.out.println(this.sexps[ptr+1].toStringShort());

    if ((this.sexps[ptr+1].ttype == tokenType) &&
	(this.sexps[ptr+1].sval.equals(s))) {
      ptr++;
      return true;
    } else
      return false;
  }
  

  public String getString()
  {
    return this.sval;
  }
  
  public String getIdent()
  {
    return this.sval;
  }
  
  public Object getObject()
  {
    return this.oval;
  }
  
  public double getNumber()
  {
    return this.nval;
  }

  // first index is 1
  public SExp getSExp(int index)
  {
    if (index > size())
      ; // throw exception
    return this.sexps[index-1];
  }

  public String errorMsg(String msg)
  {
    // example: Parse error at line 43: expect "(", get "hi"
    String s = "PICS label parse error at line " + 
      this.lineno + ":  expect :" + msg + "\n";
    try {
      this.nextToken();
    } catch (SExpException e){}
    s += ">> " + this.sexps[ptr].toStringShort() + "\n\n";
    return(s);
  } 
  
  public static void main(String[] argv)
       throws SExpException, IOException
  {
    SExp exp = new SExp();
    exp.parseMultiple(new FileInputStream(argv[0]));
    System.out.println(exp.toString());
    for (int i=1; i <= exp.size(); i+=2) {
      try {
	if (exp.getSExp(i).restrictMatch(exp.getSExp(i+1)))
	  System.out.println("matched!");
	else
	  System.out.println("not matched!");
      } catch (SExpException e) {
	System.out.println(e.getMessage());
      }
    }
  }
}

