/*
 *  PICSLabel.java
 *
 *  Copyright 1997 Massachusetts Institute of Technology.
 *  All Rights Reserved.
 *
 *  Author: Ora Lassila
 *
 *  $Id: PICSLabel.java,v 1.1 1997/04/15 20:39:26 yhchu Exp $
 */

package w3c.www.pics;

import java.util.Dictionary;
import java.util.Hashtable;
import java.util.Date;
import java.util.Vector;
import java.util.Enumeration;
import java.net.URL;
import java.io.PrintStream;
import w3c.tools.sexpr.*;

public class PICSLabel implements SExpr {

  private Symbol version;
  private URL service;
  private Dictionary options;
  private Dictionary ratings;

  public PICSLabel(Symbol version, URL service, Dictionary options)
  {
    this.version = version;
    this.service = service;
    this.options = options;
    this.ratings = new Hashtable();
  }

  public Symbol getVersion()
  {
    return version;
  }

  public URL getService()
  {
    return service;
  }

  public Object getOption(Symbol name)
  {
    return options.get(name);
  }

  public Object getOption(String name)
  {
    return options.get(name);
  }

  protected Object setOption(Symbol name, Object option)
  {
    return options.put(name, option);
  }

  public Object setOption(String name, Object option)
  {
    return options.put(name, option);
  }

  public Dictionary getRatings()
  {
    return ratings;
  }

  public RatingValue getRating(String transmitName)
  {
    return (RatingValue)ratings.get(transmitName);
  }

  public RatingValue setRating(String transmitName,
                                         RatingValue rating)
  {
    ratings.put(transmitName, rating);
    return rating;
  }

  public void addRatings(Vector data)
    throws PICSParserException
  {
    for (int i = 0; i < data.size(); i += 2) {
      Object values = data.elementAt(i + 1);
      RatingValue rating;
      if (values instanceof Number)      // single number
        rating = new RatingValue(((Number)values).doubleValue());
      else if (values instanceof Vector) // list of numbers
        rating = new RatingValue((Vector)values);
      else                               // none of the above
        throw new PICSParserException("Ill-structured ratings");
      setRating(((Symbol)data.elementAt(i)).toString(), rating);
    }
  }

  public String toString()
  {
    Vector label = new Vector();
    label.addElement(version);
    label.addElement(service);
    label.addElement(PICS.LABELS);
    Enumeration options = this.options.keys();
    while (options.hasMoreElements()) {
      Object optname = options.nextElement();
      Object optval = this.options.get(optname);
      if (optname instanceof String)  // check if extension
	label.addElement(PICS.EXTENSION);
      else
	label.addElement(optname);
      label.addElement(optval);
    }
    label.addElement(PICS.RATINGS);
    Vector rats = new Vector();
    label.addElement(rats);
    Enumeration ratings = this.ratings.keys();
    while (ratings.hasMoreElements()) {
      Object transmit_name = ratings.nextElement();
      Object ratval = this.ratings.get(transmit_name);
      rats.addElement(new Symbol((String)transmit_name));
      rats.addElement(ratval);
    }
    return PICSStream.toString(label);
  }

  public void printExpr(PrintStream out)
  {
    out.print(toString());
  }
}
