/*
 *  PICSStream.java
 *
 *  Copyright 1997 Massachusetts Institute of Technology.
 *  All Rights Reserved.
 *
 *  Author: Ora Lassila
 *
 *  $Id: PICSStream.java,v 1.1 1997/04/15 20:39:28 yhchu Exp $
 */

package w3c.www.pics;

import w3c.tools.sexpr.*;
import java.io.InputStream;
import java.io.PrintStream;
import java.io.PushbackInputStream;
import java.io.IOException;
import java.io.EOFException;
import java.util.Dictionary;
import java.util.Hashtable;
import java.util.Vector;
import java.net.URL;
import java.util.Date;
import w3c.www.dsig.*;
import w3c.www.dsig.util.*;

/**
 * Basic implementation of the SExprStream parser interface.
 */
public class PICSStream extends SimpleSExprStream implements SExprStream {

  public PICSStream (InputStream input)
  {
    super(input);
  }

  /**
   * Produces a printed representation of an s-expression.
   */
  public static void printExpr(Object expr, PrintStream out)
  {
    out.print(toString(expr));
  }


  /**
   * Produces a string representation of an s-expression.
   */
  public static String toString(Object expr)
  {
    if (expr == null)
      return "";
    if (expr instanceof Number)
      return expr.toString();
    if (expr instanceof String)
      return "\"" + (String)expr + "\"";
    if (expr instanceof URL)
      return "\"" + ((URL)expr).toString() + "\"";
    if (expr instanceof Date)
      return "\"" + ((Date)expr).toString() + "\"";
    if (expr instanceof UserOptionValue)
      return ((UserOptionValue)expr).toString();
    if (expr instanceof ISODate)
      return "\"" + ((ISODate)expr).toString() + "\"";
    if (expr instanceof RatingValue)
      return ((RatingValue)expr).toString();
    if (expr instanceof PICSExtension)
      return ((PICSExtension)expr).toString();
    if (expr instanceof Vector) {
      String s = "(";
      for (int i=0; i < ((Vector)expr).size(); i++) {
	if (i != 0)  s += " ";
	s += PICSStream.toString(((Vector)expr).elementAt(i));
      }
      return s += ")";
    }
    if (expr instanceof SExpr)
      return ((SExpr)expr).toString();
    // error
    return "<#unknown " + expr.toString() + ">)";
  }
	
  /**
   * Produces a normative form of the string representation of 
   * an s-expression.
   */
  public String toNormativeForm(Object v)
  {
    return toString(v);
  }
	


  public static void main(String args[])
    throws SExprParserException, IOException
  {
    SExprStream p = new PICSStream(System.in);
    Object e = p.parse();
    PICSStream.printExpr(e, System.out);
    System.out.println(PICSStream.toString(e));
    System.out.println();
  }

}
