/*
 *  UserOptionValue.java
 *
 *  Copyright 1997 Massachusetts Institute of Technology.
 *  All Rights Reserved.
 *
 *  Author: Ora Lassila
 *
 *  $Id: UserOptionValue.java,v 1.1 1997/04/15 20:39:35 yhchu Exp $
 */

package w3c.www.pics;

import java.util.Vector;
import w3c.tools.sexpr.Symbol;
import java.io.PrintStream;
import java.net.*;

public class UserOptionValue {

  private boolean optional = false;
  private Object data[] = {};
  private URL identifier;

  public UserOptionValue()
  {
  }

  public UserOptionValue(Vector parsedData)
    throws PICSParserException
  {
    Symbol key = (Symbol)parsedData.elementAt(0);
    if (key == PICS.OPTIONAL)
      this.optional = true;
    else if (key == PICS.MANDATORY)
      this.optional = false;
    else
      throw new PICSParserException("Extension must be optional or mandatory: "
                                    + key);
    try { identifier = new URL((String)parsedData.elementAt(1));
    } catch (MalformedURLException e) {
      throw new PICSParserException("Unable to parse extension URL " +
				    parsedData.elementAt(1), e); 
    }
    
    this.data = new Object[parsedData.size() - 2];
    for (int i = 2; i < parsedData.size(); i++)
      data[i - 2] = parsedData.elementAt(i);
  }

  public boolean isOptional()
  {
    return optional;
  }

  public Object[] getData()
  {
    return data;
  }

  public void setData(Object data[])
  {
    this.data = data;
  }
  
  public void setMandatory()
  {
    optional = false;
  }

  public void setOptional()
  {
    optional = true;
  }


  public void setURL(URL url)
  {
    identifier=url;
  }


  public URL getURL()
  {
    return identifier;
  }


  public void prinExpr(PrintStream out)
  {
    out.print(toString());
  }


  public String toString()
  {
    Vector option = new Vector();
    if (optional)
      option.addElement(PICS.OPTIONAL);
    else
      option.addElement(PICS.MANDATORY);
    option.addElement(identifier);
    for (int i=0; i<data.length; i++)
      option.addElement(data[i]);
    return PICSStream.toString(option);
  }
}
